%% Optimize well placement and rates at Utsira
[Gt, optim, init, history, other] = ...
    optimizeFormation('modelname'        , 'utsirafm'                      , ...
            'coarse_level'               , 3                               , ...
            'maximise_boundary_distance' , true                            , ...
            'trapfile_name'              , 'utsira_subtrap_function_3.mat' , ...
            'num_wells'                  , 10                              , ...
            'itime'                      , 50 * year                       , ...
            'mtime'                      , 3000 * year                     , ...
            'isteps'                     , 10                              , ...
            'msteps'                     , 31);

reports_init  = makeReports(Gt, [{other.initState}; init.states], ...
                            other.rock, other.fluid, init.schedule, ...
                            other.residual, other.traps, other.dh);

reports_optim = makeReports(Gt, [{other.initState}; optim.states], ...
                            other.rock, other.fluid, optim.schedule, ...
                            other.residual, other.traps, other.dh);

%% Figure 11: Plot map with the numbered wells
selectedResultsMultiplot(Gt, reports_optim, [2], ...
                         'plot_plume', false, ...
                         'plot_well_numbering', true, ...
                         'plot_distrib', false);%#ok

%% Figure 12: Compare default and optimized well rates
compareWellrates(init.schedule, optim.schedule, other.fluid.rhoGS);

%% Figure 13 and 14: Plot default and optimized results (trapping states, snapshots)

% initial rates
selectedResultsMultiplot(Gt, reports_init, [11 21 41], ...
                         'background', 'totalCO2', ...
                         'plot_traps', true);

% optimized rates
selectedResultsMultiplot(Gt, reports_optim, [11 21 41], ...
                         'background', 'totalCO2', ...
                         'plot_traps', true);

%% Figure 15: Overpressure at early injection
selectedResultsMultiplot(Gt, reports_optim, [2], ...
                         'background', 'overpressure', ...
                         'init_state', other.initState, ...
                         'plot_traps', true, 'plot_distrib', false);%#ok
